/*************** Includes *****************************************************************************************/
#include "system.h"
#include "app_IrrigationWatch.h"

/*************** Defines ******************************************************************************************/
#define LEFT_KEY        PORTAbits.RA0
#define RIGHT_KEY       PORTAbits.RA1
#define ZONE1_PIN       PORTBbits.RB4
#define ZONE2_PIN       PORTBbits.RB5
#define ZONE3_PIN       PORTBbits.RB6
#define ZONE4_PIN       PORTBbits.RB7
#define ALARM_PIN       PORTAbits.RA5
#define FLOWCTL_PIN     PORTAbits.RA4
#define DBcount 30          // number of times x 10mS key input must read HIGH to consider stable
#define GPM 1400            // conversion rate for cps to gpm
#define CONFIGURED  0xFF    // Is Configured value for eeprom
#define ZONE_LIMIT  5       // cps to add to Zone Limit when Set at Configuration
#define FLOW_LIM    0       // cps allowable Flow without Zone Active
#define ALARMCD     60      // number of Seconds Alarm condition must exist before Alarm mode
#define DSPOFFCD    90      // number of seconds with no activity for LCD Display OFF

/*************** Global Data Declarations *************************************************************************/
// declare global data variables here including buffers
eeprom uint8_t  IsConfiged;                 // 0x00 = NOT configured, 0xFF = configured
eeprom uint16_t eZ1Limit;                   // eeprom config variable for Zone 1 Flow Limit in counts per second
eeprom uint16_t eZ2Limit;                   // eeprom config variable for Zone 2 Flow Limit in counts per second
eeprom uint16_t eZ3Limit;                   // eeprom config variable for Zone 3 Flow Limit in counts per second
eeprom uint16_t eZ4Limit;                   // eeprom config variable for Zone 4 Flow Limit in counts per second
uint16_t Z1Limit;                           // RAM config variable for Zone 1 Flow Limit in counts per second
uint16_t Z2Limit;                           // RAM config variable for Zone 2 Flow Limit in counts per second
uint16_t Z3Limit;                           // RAM config variable for Zone 3 Flow Limit in counts per second
uint16_t Z4Limit;                           // RAM config variable for Zone 4 Flow Limit in counts per second
uint16_t FSCount;                           // variable to hold raw Flow Sensor count
uint16_t FScps;                             // variable to hold Flow Sensor counts per second
float    FSgpm;                             // variable to hold Flow Sensor gallons per minute
uint8_t  buf[80];                           // buffer to hold LCS Display messages
uint8_t  AlarmCD;                           // countdown counter for alarm condition
uint8_t  DspOffCD;                          // countdown counter for LCD Display OFF
bit CmdLeft;                                // Flag to indicate a Left keypress is ready for app processing
bit CmdEnter;                               // Flag to indicate a Enter keypress is ready for app processing
bit CmdRight;                               // Flag to indicate a Right keypress is ready for app processing
bit FLOWCTL_ON;                             // Flag to indicate Flow Control relay is Active to stop water flow
bit ALARM_ON;                               // Flag to indicate Alarm output is Active indicating Alarm Condition
bit TimerDone;                              // Flag for Timer1 to main program
bit DspON;                                  // Flag to indicate the LCD Display is ON

/*******************************************************************************************************************
 * Application Data
 * Summary: Holds application data
 * Description: This structure holds the application's data.
 * Remarks: Application strings and buffers are be defined outside this structure.
*******************************************************************************************************************/
typedef struct
{
    APP_IRRIGATIONWATCH_STATES state;       // The application's current state
    APP_SCREEN_STATES screen;               // The application's current screen state
    APP_MENU_STATES menu;                   // The application's currently selected menu item
} APP_IRRIGATIONWATCH_DATA;

/*******************************************************************************************************************
 * Description: This structure holds the application's data.
 * Remarks: This structure should be initialized by the APP_Initialize function.
    Application strings and buffers are be defined outside this structure.
*******************************************************************************************************************/
APP_IRRIGATIONWATCH_DATA appData;

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_Initialize(void);
 * Overview: Initializes the app
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_Initialize(void)
{
    appData.screen = APP_SCREEN_STATE_NONE;                         // set screen state to NONE
    appData.menu = APP_MENU_STATE_NONE;                             // set currently selected menu item to NONE
    if (IsConfiged == 0x00)     // check coinfig byte in eeprom to see if the app has ever been configurated
    {
        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE1;     // set APP STATE to CONFIG
    }
    else
    {
        appData.state = APP_IRRIGATIONWATCH_STATE_IDLE;             // set APP STATE to IDLE
        Z1Limit = eZ1Limit;                                         // read stored eeprom value into RAM
        Z2Limit = eZ2Limit;                                         // read stored eeprom value into RAM
        Z3Limit = eZ3Limit;                                         // read stored eeprom value into RAM
        Z4Limit = eZ4Limit;                                         // read stored eeprom value into RAM
    }
    IOCAF0_SetInterruptHandler(APP_IrrigationWatch_IOCAF0CallBack);   // set the IOCAF0 call back function
    IOCAF1_SetInterruptHandler(APP_IrrigationWatch_IOCAF1CallBack);   // set the IOCAF1 call back function
    TMR1_SetInterruptHandler(APP_IrrigationWatch_TMR1CallBack);       // set the TMR1 call back function
    DspOffCD = DSPOFFCD;            // initialize LCD Display Off Countdown Counter
    AlarmCD = ALARMCD;              // init Alarm Countdown Counter, alarm condition must exist for a while
    CmdLeft = 0;                    // set Command Left Flag to false
    CmdEnter = 0;                   // set Command Enter Flag to false
    CmdRight = 0;                   // set Command Right Flag to false
    FLOWCTL_ON = 0;                 // set FLOWCTL_ON Flag to false
    ALARM_ON = 0;                   // set ALARM_ON Flag to false
    DspON = 1;                      // set Display ON Flag to true
    LCD_Initialize(LCD_COLS, LCD_LINES);                              // init the LCD Display
    BacklightOn();                  // turn ON LCD Display
    FSCount = 0;                    // init Flow Sensor count
    FScps = 0;                      // init Flow Sensor counts per second
    TMR1_StartTimer();              // Start TMR1 timmer
}
 
/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_Tasks(void);
 * Overview: perform application tasks as needed
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_Tasks(void)
{
    // Process Command Keypress
    APP_IrrigationWatch_ProcessKeyPress();
    
    // Process Screen Display and Menu Choice
    APP_IrrigationWatch_ProcessDisplayMenu();
    
    // Process Displayed values and Flow Limits check every 1 Second
    if (TimerDone == 1)
    {
        // Process Displayed values
        APP_IrrigationWatch_ProcessDisplayValues();
    
        // Check Flow Limits and Alarm if exceeded
        APP_IrrigationWatch_MonitorLimits();
        
        // decrement Display OFF Countdown Counter, turn OFF display if ZERO
        if ((DspON == 1) && (appData.state == APP_IRRIGATIONWATCH_STATE_MONITOR))
        {
            DspOffCD--;                 // decrement Display OFF Countdown Counter
            if (DspOffCD == 0)
            {
                BacklightOff();         // turn OFF LCD Display
                DspON = 0;              // Set Display ON Flag to false
            }
        }
        // toggle alarm output if ON for 1 sec beep of alarm annunciator
        if (ALARM_ON == 1)
        {
            if (ALARM_PIN == 1)
            {
                ALARM_PIN = 0;
            }
            else
            {
                ALARM_PIN = 1;
            }
        }
        
        // reset Timer Flag
        TimerDone = 0;
    }
}

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_ProcessKeyPress(void);
 * Overview: perform tasks from key press if needed
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_ProcessKeyPress(void)
{
    // if LCD Display is OFF then turn ON and clear keypress, this one just wakes up display
    if ((DspON == 0) && ((CmdLeft == 1) || (CmdEnter == 1) || (CmdRight == 1)))
    {
        BacklightOn();          // turn ON LCD Display
        DspON = 1;              // Set Display ON Flag to true
        DspOffCD = DSPOFFCD;    // reset LCD Display OFF Countdown Counter
        // clear all key command flags
        CmdLeft = 0;
        CmdEnter = 0;
        CmdRight = 0;
    }
    // Process Left Key Press
    if (CmdLeft == 1)
    {
        DspOffCD = DSPOFFCD;        // reset LCD Display Off Countdown Counter
        switch (appData.menu)
        {
            case APP_MENU_STATE_NONE:
            {
                appData.state = APP_IRRIGATIONWATCH_STATE_IDLE;     // set app state to IDLE
                appData.screen = APP_SCREEN_STATE_NONE;             // set screen state to NONE
                break;
            }
            case APP_MENU_STATE_MONITOR:
            {
                break;                                      // don't do anything, no menu item to Left
            }
            case APP_MENU_STATE_CONFIG:
            {
                SetCursorPos(1, 2);                         // blink MONITOR menu choice
                appData.menu = APP_MENU_STATE_MONITOR;      // set menu state to MONITOR
                break;
            }
            case APP_MENU_STATE_ZONESET:
            {
                break;                                      // don't do anything, no menu item to Left
            }
            case APP_MENU_STATE_ZONENEXT:
            {
                SetCursorPos(1, 2);                         // blink SET menu choice
                appData.menu = APP_MENU_STATE_ZONESET;      // set menu state to ZONESET
                break;
            }
            case APP_MENU_STATE_ZONEDISABLE:
            {
                SetCursorPos(5, 2);                         // blink NEXT menu choice
                appData.menu = APP_MENU_STATE_ZONENEXT;     // set menu state to NEXT
                break;
            }
            case APP_MENU_STATE_RESET:
            {
                break;                                      // don't do anything, no menu item to Left
            }
            case APP_MENU_STATE_BYPASS:
            {
                SetCursorPos(1, 2);                         // blink RESET menu choice
                appData.menu = APP_MENU_STATE_RESET;        // set menu state to RESET
                break;
            }
        }
        CmdLeft = 0;                // reset Command Flag
    }
    // Process Enter Key Press
    if (CmdEnter == 1)
    {
        DspOffCD = DSPOFFCD;        // reset LCD Display Off Countdown Counter
        switch (appData.menu)
        {
            case APP_MENU_STATE_NONE:
            {
                appData.state = APP_IRRIGATIONWATCH_STATE_IDLE;         // set app state to IDLE
                appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                break;
            }
            case APP_MENU_STATE_MONITOR:
            {
                appData.state = APP_IRRIGATIONWATCH_STATE_MONITOR;      // set app state to MONITOR
                appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                break;
            }
            case APP_MENU_STATE_CONFIG:
            {
                appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE1; // set app state to CONFIG_ZONE1
                appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                break;
            }
            case APP_MENU_STATE_ZONESET:
            {
                switch (appData.state)
                {
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE1:
                    {
                        // set Zone 1 Limit
                        Z1Limit = (FScps + ZONE_LIMIT);                         // save Zone 1 Flow Limit
                        eZ1Limit = Z1Limit;                                     // update Z1 Flow Limit in eeprom
                        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE2; // set app state to CONFIG_ZONE2
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE2:
                    {
                        // set Zone 2 Limit
                        Z2Limit = (FScps + ZONE_LIMIT);                         // save Zone 2 Flow Limit
                        eZ2Limit = Z2Limit;                                     // update Z2 Flow Limit in eeprom
                        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE3; // set app state to CONFIG_ZONE3
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE3:
                    {
                        // set Zone 3 Limit
                        Z3Limit = (FScps + ZONE_LIMIT);                         // save Zone 3 Flow Limit
                        eZ3Limit = Z3Limit;                                     // update Z3 Flow Limit in eeprom
                        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE4; // set app state to CONFIG_ZONE4
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE4:
                    {
                        // set Zone 4 Limit
                        Z4Limit = (FScps + ZONE_LIMIT);                         // save Zone 4 Flow Limit
                        eZ4Limit = Z4Limit;                                     // update Z4 Flow Limit in eeprom
                        appData.state = APP_IRRIGATIONWATCH_STATE_IDLE;         // set app state to IDLE
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                }
                if ((Z1Limit>0) || (Z2Limit>0) || (Z3Limit>0) || (Z4Limit>0))
                {
                    IsConfiged = CONFIGURED;        // set IsConfiged to indicate on pgm startup app is configed
                }
                break;
            }
            case APP_MENU_STATE_ZONENEXT:
            {
                switch (appData.state)
                {
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE1:
                    {
                        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE2; // set app state to CONFIG_ZONE2
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE2:
                    {
                        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE3; // set app state to CONFIG_ZONE3
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE3:
                    {
                        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE4; // set app state to CONFIG_ZONE4
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE4:
                    {
                        appData.state = APP_IRRIGATIONWATCH_STATE_IDLE;     // set app state to IDLE
                        appData.screen = APP_SCREEN_STATE_NONE;             // set screen state to NONE
                        break;
                    }
                }
                break;
            }
            case APP_MENU_STATE_ZONEDISABLE:
            {
                switch (appData.state)
                {
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE1:
                    {
                        Z1Limit = 0;                                            // disable Zone 1
                        eZ1Limit = 0;                                           // update Z1 Flow Limit in eeprom
                        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE2; // set app state to CONFIG_ZONE2
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE2:
                    {
                        Z2Limit = 0;                                            // disable Zone 2
                        eZ2Limit = 0;                                           // update Z2 Flow Limit in eeprom
                        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE3; // set app state to CONFIG_ZONE3
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE3:
                    {
                        Z3Limit = 0;                                            // disable Zone 3
                        eZ3Limit = 0;                                           // update Z3 Flow Limit in eeprom
                        appData.state = APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE4; // set app state to CONFIG_ZONE4
                        appData.screen = APP_SCREEN_STATE_NONE;                 // set screen state to NONE
                        break;
                    }
                    case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE4:
                    {
                        Z4Limit = 0;                                        // disable Zone 4
                        eZ4Limit = 0;                                       // update Zone 4 Flow Limit in eeprom
                        appData.state = APP_IRRIGATIONWATCH_STATE_IDLE;     // set app state to IDLE
                        appData.screen = APP_SCREEN_STATE_NONE;             // set screen state to NONE
                        break;
                    }
                }
                break;
            }
            case APP_MENU_STATE_RESET:
            {
                AlarmCD = ALARMCD;                              // resent Alarm Countdown counter
                FLOWCTL_ON = 0;                                 // set FLOWCTL_ON Flag to false
                ALARM_ON = 0;                                   // set ALARM_ON Flag to false
                ALARM_PIN = 1;                                         // Alarm Output HIGH turn OFF annunciator
                FLOWCTL_PIN = 1;                                       // Flow Control HIGH to ALLOW water flow
                appData.state = APP_IRRIGATIONWATCH_STATE_IDLE;        // set app state to IDLE
                appData.screen = APP_SCREEN_STATE_NONE;                // set screen state to NONE
                break;
            }
            case APP_MENU_STATE_BYPASS:
            {
                // clear Flow Control ON Condition
                FLOWCTL_ON = 0;                                  // set FLOWCTL_ON Flag to false
                FLOWCTL_PIN = 1;                                 // Flow Control HIGH to ALLOW water flow
                appData.screen = APP_SCREEN_STATE_NONE;          // set screen state to NONE
                break;
            }
        }
        CmdEnter = 0;               // reset Command Flag
    }
    // Process Right Key Press
    if (CmdRight == 1)
    {
        DspOffCD = DSPOFFCD;        // reset LCD Display Off Countdown Counter
        switch (appData.menu)
        {
            case APP_MENU_STATE_NONE:
            {
                appData.state = APP_IRRIGATIONWATCH_STATE_IDLE;     // set app state to IDLE
                appData.screen = APP_SCREEN_STATE_NONE;             // set screen state to NONE
                break;
            }
            case APP_MENU_STATE_MONITOR:
            {
                SetCursorPos(11, 2);                        // blink CONFIG menu choice
                appData.menu = APP_MENU_STATE_CONFIG;       // set menu state to CONFIG
                break;
            }
            case APP_MENU_STATE_CONFIG:
            {
                break;                                      // don't do anything, no menu item to Right
            }
            case APP_MENU_STATE_ZONESET:
            {
                SetCursorPos(5, 2);                         // blink CONFIG menu choice
                appData.menu = APP_MENU_STATE_ZONENEXT;     // set menu state to ZONENEXT
                break;
            }
            case APP_MENU_STATE_ZONENEXT:
            {
                SetCursorPos(10, 2);                        // blink CONFIG menu choice
                appData.menu = APP_MENU_STATE_ZONEDISABLE;  // set menu state to ZONEDISABLE
                break;
            }
            case APP_MENU_STATE_ZONEDISABLE:
            {
                break;                                      // don't do anything, no menu item to Right
            }
            case APP_MENU_STATE_RESET:
            {
                SetCursorPos(11, 2);                        // blink CONFIG menu choice
                appData.menu = APP_MENU_STATE_BYPASS;       // set menu state to BYPASS
                break;
            }
            case APP_MENU_STATE_BYPASS:
            {
                break;                                      // don't do anything, no menu item to Right
            }
        }
        CmdRight = 0;               // reset Command Flag
    }
}

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_ProcessDisplayMenu(void)
 * Overview: perform tasks for the display and menu choices
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_ProcessDisplayMenu(void)
{
    switch (appData.state)
    {
        case APP_IRRIGATIONWATCH_STATE_IDLE:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                strcpy(buf, "Irrigation Watch");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Monitor   Config");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_IDLE;
                appData.menu = APP_MENU_STATE_MONITOR;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_MONITOR:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                BlinkOff();
                ClearDisplay();
                // reset LCD Countdown Counter and make sure Display is ON
                BacklightOn();          // turn ON LCD Display
                DspON = 1;              // Set Display ON Flag to true
                DspOffCD = DSPOFFCD;    // reset LCD Display OFF Countdown Counter
                strcpy(buf, "Active Zone None");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "FlowRate 0.0 gpm");
                WriteDataMsg(buf, strlen(buf));
                appData.screen = APP_SCREEN_STATE_MONITOR;
                appData.menu = APP_MENU_STATE_NONE;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_ZONE1:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                // reset LCD Countdown Counter and make sure Display is ON
                BacklightOn();          // turn ON LCD Display
                DspON = 1;              // Set Display ON Flag to true
                DspOffCD = DSPOFFCD;    // reset LCD Display OFF Countdown Counter
                strcpy(buf, "Alarm Z1     gpm");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Reset     Bypass");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_ALARM;
                appData.menu = APP_MENU_STATE_RESET;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_ZONE2:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                // reset LCD Countdown Counter and make sure Display is ON
                BacklightOn();          // turn ON LCD Display
                DspON = 1;              // Set Display ON Flag to true
                DspOffCD = DSPOFFCD;    // reset LCD Display OFF Countdown Counter
                strcpy(buf, "Alarm Z2     gpm");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Reset     Bypass");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_ALARM;
                appData.menu = APP_MENU_STATE_RESET;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_ZONE3:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                // reset LCD Countdown Counter and make sure Display is ON
                BacklightOn();          // turn ON LCD Display
                DspON = 1;              // Set Display ON Flag to true
                DspOffCD = DSPOFFCD;    // reset LCD Display OFF Countdown Counter
                strcpy(buf, "Alarm Z3     gpm");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Reset     Bypass");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_ALARM;
                appData.menu = APP_MENU_STATE_RESET;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_ZONE4:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                // reset LCD Countdown Counter and make sure Display is ON
                BacklightOn();          // turn ON LCD Display
                DspON = 1;              // Set Display ON Flag to true
                DspOffCD = DSPOFFCD;    // reset LCD Display OFF Countdown Counter
                strcpy(buf, "Alarm Z4     gpm");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Reset     Bypass");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_ALARM;
                appData.menu = APP_MENU_STATE_RESET;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_FLOW:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                // reset LCD Countdown Counter and make sure Display is ON
                BacklightOn();          // turn ON LCD Display
                DspON = 1;              // Set Display ON Flag to true
                DspOffCD = DSPOFFCD;    // reset LCD Display OFF Countdown Counter
                strcpy(buf, "AlarmFLm     gpm");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Reset     Bypass");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_ALARM;
                appData.menu = APP_MENU_STATE_RESET;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE1:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                strcpy(buf, "Z1 Flow=     gpm");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Set Next Disable");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_CONFIG_ZONE1;
                appData.menu = APP_MENU_STATE_ZONESET;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE2:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                strcpy(buf, "Z2 Flow=     gpm");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Set Next Disable");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_CONFIG_ZONE2;
                appData.menu = APP_MENU_STATE_ZONESET;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE3:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                strcpy(buf, "Z3 Flow=     gpm");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Set Next Disable");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_CONFIG_ZONE3;
                appData.menu = APP_MENU_STATE_ZONESET;
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE4:
        {
            if (appData.screen == APP_SCREEN_STATE_NONE)
            {
                ClearDisplay();
                strcpy(buf, "Z4 Flow=     gpm");
                WriteDataMsg(buf, strlen(buf));
                strcpy(buf, "Set Next Disable");
                WriteDataMsg(buf, strlen(buf));
                SetCursorPos(1, 2);
                BlinkOn();
                appData.screen = APP_SCREEN_STATE_CONFIG_ZONE4;
                appData.menu = APP_MENU_STATE_ZONESET;
            }
        }
    }
}

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_ProcessKeyPress(void);
 * Overview: perform tasks for the display of Flow Rates and Active Zones
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_ProcessDisplayValues(void)
{
    
    double f;
    int status;
    char* c;
    uint8_t addr = GetDDRAMAddr();      // get and save current Diplay RAM Address Counter
        
    f = (round((FSgpm*10))/10);
    if (f > 9.9)  {   f = 9.9;    }     // limit gpm to 9.9
    c = ftoa(f, &status);
    strcpy(buf, c);
    buf[3] = 0x00;
    switch (appData.state)
    {
        case APP_IRRIGATIONWATCH_STATE_MONITOR:
        {
            // Display Flow Rate
            SetCursorPos(10, 2);
            WriteDataMsg(buf, strlen(buf));
            // display what Zone is Active
            if (ZONE1_PIN == 0)
            {
                SetCursorPos(8, 1);
                strcpy(buf, "Zone 1   ");
                WriteDataMsg(buf, strlen(buf));
            }
            if (ZONE2_PIN == 0)
            {
                SetCursorPos(8, 1);
                strcpy(buf, "Zone 2   ");
                WriteDataMsg(buf, strlen(buf));
            }
            if (ZONE3_PIN == 0)
            {
                SetCursorPos(8, 1);
                strcpy(buf, "Zone 3   ");
                WriteDataMsg(buf, strlen(buf));
            }
            if (ZONE4_PIN == 0)
            {
                SetCursorPos(8, 1);
                strcpy(buf, "Zone 4   ");
                WriteDataMsg(buf, strlen(buf));
            }
            // No Active Zones
            if ((ZONE1_PIN == 1) && (ZONE2_PIN == 1) && (ZONE3_PIN == 1) && (ZONE4_PIN == 1))
            {
                SetCursorPos(8, 1);
                strcpy(buf, "Zone None");
                WriteDataMsg(buf, strlen(buf));
            }
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_ZONE1:
        {
            // Display Flow Rate
            SetCursorPos(10, 1);
            WriteDataMsg(buf, strlen(buf));
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_ZONE2:
        {
            // Display Flow Rate
            SetCursorPos(10, 1);
            WriteDataMsg(buf, strlen(buf));
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_ZONE3:
        {
            // Display Flow Rate
            SetCursorPos(10, 1);
            WriteDataMsg(buf, strlen(buf));
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_ZONE4:
        {
            // Display Flow Rate
            SetCursorPos(10, 1);
            WriteDataMsg(buf, strlen(buf));
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_ALARM_FLOW:
        {
            // Display Flow Rate
            SetCursorPos(10, 1);
            WriteDataMsg(buf, strlen(buf));
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE1:
        {
            SetCursorPos(10, 1);
            WriteDataMsg(buf, strlen(buf));
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE2:
        {
            SetCursorPos(10, 1);
            WriteDataMsg(buf, strlen(buf));
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE3:
        {
            SetCursorPos(10, 1);
            WriteDataMsg(buf, strlen(buf));
            break;
        }
        case APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE4:
        {
            SetCursorPos(10, 1);
            WriteDataMsg(buf, strlen(buf));
            break;
        }
    }
    SetDDRAMAddr(addr);         // restore previous Display Address Counter
}

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_ProcessKeyPress(void);
 * Overview: perform tasks at monitor Flow Limits for Zones and Flow without Zone Active
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_MonitorLimits(void)
{
    if (appData.state == APP_IRRIGATIONWATCH_STATE_MONITOR)
    {
        // check Flow without Zone Active
        if ((FScps > FLOW_LIM) && (ZONE1_PIN == 1) && (ZONE2_PIN == 1) && (ZONE3_PIN == 1) && (ZONE4_PIN == 1))
        {
            AlarmCD--;
            if (AlarmCD == 0)
            {
                FLOWCTL_ON = 1;                                             // set FLOWCTL_ON Flag to true
                ALARM_ON = 1;                                               // set ALARM_ON Flag to true
                appData.state = APP_IRRIGATIONWATCH_STATE_ALARM_FLOW;       // set APP STATE to ALARM Flow Limit
                appData.screen = APP_SCREEN_STATE_NONE;                     // set screen state to NONE
            }
            else    AlarmCD = ALARMCD;                  // re-init Alarm Condition countdown counter
        }
        // check each Zone for Active AND Flow exceeding Limit
        if (ZONE1_PIN == 0)
        {
            if ((Z1Limit > 0) && (FScps > Z1Limit))     // Zone1 enabled with limit AND exceeded
            {
                AlarmCD--;
                if (AlarmCD == 0)
                {
                    FLOWCTL_ON = 1;                                             // set FLOWCTL_ON Flag to true
                    ALARM_ON = 1;                                               // set ALARM_ON Flag to true
                    appData.state = APP_IRRIGATIONWATCH_STATE_ALARM_ZONE1;      // set APP STATE to ALARM Zone 1
                    appData.screen = APP_SCREEN_STATE_NONE;                     // set screen state to NONE
                }
            }
            else    AlarmCD = ALARMCD;                  // re-init Alarm Condition countdown counter
        }
        if (ZONE2_PIN == 0)
        {
            if ((Z2Limit > 0) && (FScps > Z2Limit))     // Zone2 enabled with limit AND exceeded
            {
                AlarmCD--;
                if (AlarmCD == 0)
                {
                    FLOWCTL_ON = 1;                                             // set FLOWCTL_ON Flag to true
                    ALARM_ON = 1;                                               // set ALARM_ON Flag to true
                    appData.state = APP_IRRIGATIONWATCH_STATE_ALARM_ZONE2;      // set APP STATE to ALARM Zone 2
                    appData.screen = APP_SCREEN_STATE_NONE;                     // set screen state to NONE
                }
            }
            else    AlarmCD = ALARMCD;                  // re-init Alarm Condition countdown counter
        }
        if (ZONE3_PIN == 0)
        {
            if ((Z3Limit > 0) && (FScps > Z3Limit))     // Zone3 enabled with limit AND exceeded
            {
                AlarmCD--;
                if (AlarmCD == 0)
                {
                    FLOWCTL_ON = 1;                                             // set FLOWCTL_ON Flag to true
                    ALARM_ON = 1;                                               // set ALARM_ON Flag to true
                    appData.state = APP_IRRIGATIONWATCH_STATE_ALARM_ZONE3;      // set APP STATE to ALARM Zone 3
                    appData.screen = APP_SCREEN_STATE_NONE;                     // set screen state to NONE
                }
            }
            else    AlarmCD = ALARMCD;                  // re-init Alarm Condition countdown counter
        }
        if (ZONE4_PIN == 0)
        {
            if ((Z4Limit > 0) && (FScps > Z4Limit))     // Zone4 enabled with limit AND exceeded
            {
                AlarmCD--;
                if (AlarmCD == 0)
                {
                    FLOWCTL_ON = 1;                                             // set FLOWCTL_ON Flag to true
                    ALARM_ON = 1;                                               // set ALARM_ON Flag to true
                    appData.state = APP_IRRIGATIONWATCH_STATE_ALARM_ZONE4;      // set APP STATE to ALARM Zone 4
                    appData.screen = APP_SCREEN_STATE_NONE;                     // set screen state to NONE
                }
            }
            else    AlarmCD = ALARMCD;                  // re-init Alarm Condition countdown counter
        }
        if (ALARM_ON == 1)  {   ALARM_PIN = 0; }        // Alarm Output LOW turn ON
        if (FLOWCTL_ON == 1)  {   FLOWCTL_PIN = 0; }    // Flow Control Output LOW to STOP water flow
        if ((ZONE1_PIN == 0) || (ZONE2_PIN == 0) || (ZONE3_PIN == 0) || (ZONE4_PIN == 0))
        {
            DspOffCD = DSPOFFCD;        // reset LCD Display Off Countdown Counter
            if (DspON == 0)             // check if display is OFF and if so, turn ON LCD Display
            {
                BacklightOn();          // turn ON LCD Display
                DspON = 1;              // Set Display ON Flag to true
            }
        }
    }
}

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_INTPin(void);
 * Overview: process Flow Sensor interrupts
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_INTPin(void)
{
     FSCount++;                                             // increment Flow Sensor count
     INTCONbits.INTF = 0;                                   // clear INT Interrupt Flag
}

/*******************************************************************************************************************
 * Function: APP_IrrigationWatch_TMR1CallBack(void);
 * Overview: process TMR1 call backs every 1 sec
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_TMR1CallBack(void)
{
    FScps = FSCount;
    if (FScps>280)  FScps = 280;        // limit Flow Sensor Count to less than 9.9 gpm
    FSgpm = (((float)FScps*60)/GPM);    // convert counts per second to gallons per minute
    FSCount = 0;                        // reset Flow Sensor Counter
    TimerDone = 1;                      // set Flag for main program to update display
}
 
/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_IOCAF0CallBack(void);
 * Overview: process IOCAF0 call back DBcount
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_IOCAF0CallBack(void)
{
    IOCANbits.IOCAN0 = 0;               // disable any more negative edge triggers until we are done here
    if (IOCAFbits.IOCAF1 == 1)
    {
        CmdEnter = 1;           // set Command Enter Flag to true
        IOCANbits.IOCAN1 = 0;           // disable any more negative edge triggers until we are done here 
        IOCAFbits.IOCAF1 = 0;           // clear IOCAF1 Flag bit, we don't need to service IOCAF1
    }
    else
    {
        CmdLeft = 1;            // set Command Left Flag to true
    }
    // switch de-bounce
    for (uint8_t x=0; x<DBcount; x++)
    {
        do
        {
            __delay_ms(10);                 // delay 10mS
        } while (LEFT_KEY == 0);
    }   // wait for LEFT_KEY to go back HIGH and stay HIGH for at least DBcount x 10mS
    IOCANbits.IOCAN0 = 1;               // re-enable negative edge triggers again
    IOCANbits.IOCAN1 = 1;               // re-enable negative edge triggers again
}

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_IOCAF1CallBack(void);
 * Overview: process IOCAF1 call back
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_IOCAF1CallBack(void)
{
    IOCANbits.IOCAN1 = 0;               // disable any more negative edge triggers until we are done here
    if (IOCAFbits.IOCAF0 == 1)
    {
        CmdEnter = 1;           // set Command Enter Flag to true
        IOCANbits.IOCAN0 = 0;           // disable any more negative edge triggers until we are done here 
        IOCAFbits.IOCAF0 = 0;           // clear IOCAF0 Flag bit, we don't need to service IOCAF0
    }
    else
    {
        CmdRight = 1;       // set Command Right Flag to true
    }
    // switch de-bounce
    for (uint8_t x=0; x<DBcount; x++)
    {
        do
        {
            __delay_ms(10);                 // delay 10mS
        } while (LEFT_KEY == 0);
    }   // wait for LEFT_KEY to go back HIGH and stay HIGH for at least DBcount x 10mS
    IOCANbits.IOCAN0 = 1;               // re-enable negative edge triggers again
    IOCANbits.IOCAN1 = 1;               // re-enable negative edge triggers again
}
/*************** End of File **************************************************************************************/