#ifndef APP_IRRIGATIONWATCH_H
#define APP_IRRIGATIONWATCH_H

/*******************************************************************************************************************
 * LCD Display Select - Define Number of characters and lines of the display 
*******************************************************************************************************************/
#define LCD_COLS 16         // number of cols (characters) of the LCD Display
#define LCD_LINES 2         // number of lines of the LCD Display

/*******************************************************************************************************************
 * LCD Display Interface Select - I/O Expander or I/O Port (4 Control lines and 4 Data lines)
 * Note: User MUST only define ONE of these interfaces LCD_IOEXPANDERx or LCD_PORT and
 * if using I/O Expander user must define it's I2C Address
*******************************************************************************************************************/
//#define LCD_IOEXPANDER1     // using MSSP1 comment out MSSP module NOT used
//#define LCD_IOEXPANDER2     // using MSSP2
//#define LCD_ADDR 0x27       // I2C Address of the I/O Expander
#define LCD_IOPORT            // commented out to use I/O Expander interface

#ifdef LCD_IOEXPANDER1
#include "io_expander.h"
#include "i2c1.h"
#endif

#ifdef LCD_IOEXPANDER2
#include "io_expander.h"
#include "i2c2.h"
#endif

#ifdef LCD_IOPORT
#include "IO_Port.h"
#define LCD_PORT        PORTC
#define LCD_LAT         LATC
#endif

/*******************************************************************************************************************
 * Application states
 * Summary: Application states enumeration
 * Description:
    This enumeration defines the valid application states.  These states
    determine the behavior of the application at various times.
*******************************************************************************************************************/
typedef enum
{
    APP_IRRIGATIONWATCH_STATE_IDLE,               // APP STATE when app is not in Monitor, Alarm or Config state
    APP_IRRIGATIONWATCH_STATE_MONITOR,            // APP STATE when app is monitoring the irrigation system
    APP_IRRIGATIONWATCH_STATE_ALARM_ZONE1,        // APP STATE when app in ALARM Zone 1
    APP_IRRIGATIONWATCH_STATE_ALARM_ZONE2,        // APP STATE when app in ALARM Zone 2
    APP_IRRIGATIONWATCH_STATE_ALARM_ZONE3,        // APP STATE when app in ALARM Zone 3
    APP_IRRIGATIONWATCH_STATE_ALARM_ZONE4,        // APP STATE when app in ALARM Zone 4
    APP_IRRIGATIONWATCH_STATE_ALARM_FLOW,         // APP STATE when app in ALARM Flow w/o Zone Active
    APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE1,       // APP STATE when user is configuring Zone 1 Limit
    APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE2,       // APP STATE when user is configuring Zone 2 Limit
    APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE3,       // APP STATE when user is configuring Zone 3 Limit
    APP_IRRIGATIONWATCH_STATE_CONFIG_ZONE4        // APP STATE when user is configuring Zone 4 Limit
} APP_IRRIGATIONWATCH_STATES;

/*******************************************************************************************************************
 * Screen states
 * Summary:     Application screen states enumeration
 * Description: These enumeration define what screen is display at the present time.
*******************************************************************************************************************/
typedef enum
{
    APP_SCREEN_STATE_NONE,              // APP SCREEN STATE when screen has nothing displayed yet
    APP_SCREEN_STATE_IDLE,              // APP SCREEN STATE when screen has Idle/Welcome Screen displayed
    APP_SCREEN_STATE_MONITOR,           // APP SCREEN STATE when screen has Monitor Screen displayed
    APP_SCREEN_STATE_ALARM,             // APP SCREEN STATE when screen has Alarm Screen displayed
    APP_SCREEN_STATE_CONFIG_ZONE1,      // APP SCREEN STATE when screen has Zone 1 Limit Config Screen displayed
    APP_SCREEN_STATE_CONFIG_ZONE2,      // APP SCREEN STATE when screen has Zone 2 Limit Config Screen displayed
    APP_SCREEN_STATE_CONFIG_ZONE3,      // APP SCREEN STATE when screen has Zone 3 Limit Config Screen displayed
    APP_SCREEN_STATE_CONFIG_ZONE4       // APP SCREEN STATE when screen has Zone 4 Limit Config Screen displayed
} APP_SCREEN_STATES;

/*******************************************************************************************************************
 * Menu states
 * Summary:     Application menu states enumeration
 * Description: These enumeration define what menu is default at the present time.
*******************************************************************************************************************/
typedef enum
{
    APP_MENU_STATE_NONE,                // APP MENU STATE when Nothing is currently selected in a menu
    APP_MENU_STATE_MONITOR,             // APP MENU STATE when Monitor is currently selected in the menu
    APP_MENU_STATE_CONFIG,              // APP MENU STATE when Config is currently selected in the menu
    APP_MENU_STATE_ZONESET,             // APP MENU STATE when a Zone Limit is currently selected in the menu
    APP_MENU_STATE_ZONENEXT,            // APP MENU STATE when Next Zone Limit is currently selected in the menu
    APP_MENU_STATE_ZONEDISABLE,         // APP MENU STATE when Zone Disable is currently selected in the menu
    APP_MENU_STATE_RESET,               // APP MENU STATE when Reset is currently selected in the menu
    APP_MENU_STATE_BYPASS               // APP MENU STATE when Bypass is currently selected in the menu
} APP_MENU_STATES;



/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_Initialize(void);
 * Overview: Initializes the app
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_Initialize(void);

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_Tasks(void);
 * Overview: perform application tasks as needed
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_Tasks(void);
 
/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_ProcessKeyPress(void);
 * Overview: perform tasks from key press if needed
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_ProcessKeyPress(void);

/*******************************************************************************************************************
 * Function: vvoid APP_IrrigationWatch_ProcessDisplayMenu(void)
 * Overview: perform tasks for the display and menu choices
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_ProcessDisplayMenu(void);

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_ProcessKeyPress(void);
 * Overview: perform tasks for the display of Flow Rates and Active Zones
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_ProcessDisplayValues(void);

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_ProcessKeyPress(void);
 * Overview: perform tasks at monitor Flow Limits for Zones and Flow without Zone Active
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_MonitorLimits(void);

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_INTPin(void);
 * Overview: process Flow Sensor interrupts
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_INTPin(void);

/*******************************************************************************************************************
 * Function: APP_IrrigationWatch_TMR1CallBack(void);
 * Overview: process TMR1 call backs every 1 sec
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_TMR1CallBack(void);

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_IOCAF0CallBack(void);
 * Overview: process IOCAF0 call back
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_IOCAF0CallBack(void);

/*******************************************************************************************************************
 * Function: void APP_IrrigationWatch_IOCAF1CallBack(void);
 * Overview: process IOCAF1 call back
 * PreCondition: None
*******************************************************************************************************************/
void APP_IrrigationWatch_IOCAF1CallBack(void);

#endif  /* APP_IRRIGATIONWATCH_H */
/*************** End of File **************************************************************************************/